/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "timeout.h"

int lobj_timeout_add(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TFUNCTION);
	
	#ifdef IDEBUG
	g_debug("Adding a timeout!");
	#endif
		
	/* Ref the function and the user data */
	int ud_ref = -1;
	
	if(lua_gettop(L) >= 4)
	{
		ud_ref = luaL_ref(L, LUA_REGISTRYINDEX);
		lua_pushvalue(L, 3);
	}	
	
	int function_ref = luaL_ref(L, LUA_REGISTRYINDEX);	
	
	/* Create the callback data */
	Data* data = g_malloc(sizeof(Data));
	data->L = L;
	data->function_ref = function_ref;
	data->ud_ref = ud_ref;
	data->signal_info.param_types = NULL;
	data->signal_info.n_params = 0;
	data->signal_info.return_type = G_TYPE_BOOLEAN;
	
	/* Connect and send the id to Lua */
	guint res = priv_timeout_add(lua_tointeger(L, 1), lua_tointeger(L, 2), data);
	lua_pushnumber(L, res);
	
	return 1;
}

int lobj_timeout_remove(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	
	#ifdef IDEBUG
	g_debug("Removing a timeout!");
	#endif
	
	guint id = (guint)lua_tointeger(L, 1);
	g_source_remove(id);
	
	return 0; 
}

int lobj_timeout_idle_add(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	luaL_checktype(L, 2, LUA_TFUNCTION);
	
	#ifdef IDEBUG
	g_debug("Adding a idle timeout!");
	#endif
		
	/* Ref the function and the user data */
	int ud_ref = -1;
	
	if(lua_gettop(L) >= 3)
	{
		ud_ref = luaL_ref(L, LUA_REGISTRYINDEX);
		lua_pushvalue(L, 2);
	}	
	
	int function_ref = luaL_ref(L, LUA_REGISTRYINDEX);	
	
	/* Create the callback data */
	Data* data = g_malloc(sizeof(Data));
	data->L = L;
	data->function_ref = function_ref;
	data->ud_ref = ud_ref;
	data->signal_info.param_types = NULL;
	data->signal_info.n_params = 0;
	data->signal_info.return_type = G_TYPE_BOOLEAN;
	
	/* Connect and send the id to Lua */
	guint res = priv_idle_add(lua_tointeger(L, 1), data);
	lua_pushnumber(L, res);
	
	return 1;
}
